//+------------------------------------------------------------------+
//|                                                        MABOL.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <ExpertAdvisor.mqh>

input int    SL        =  50; //  
input int    TP        = 100; //  
input int    TS        =  50; //   
input int    MAPer     =  12; //  MA
input int    BBPer     =  20; //  Bollinger Bands
input double BBDev     =   2; //  Bollinger Bands
input double Risk      = 0.1; // 

//---
class CMyEA : public CExpertAdvisor
  {
protected:
   double            m_risk;          //  
   int               m_sl;            // 
   int               m_tp;            // 
   int               m_ts;            // 
   int               m_MAPer;         //  MA
   int               m_BBPer;         //  Bollinger Bands     
   double            m_BBDev;         //  Bollinger Bands     
   int               m_hma;           //   
   int               m_hbb;           //   Bollinger Bands   
public:
   void              CMyEA();
   void             ~CMyEA();
   virtual bool      Init(string smb,ENUM_TIMEFRAMES tf); // 
   virtual bool      Main();                              //  
   virtual void      OpenPosition(long dir);              //    
   virtual void      ClosePosition(long dir);             //    
   virtual long      CheckSignal(bool bEntry);            //  
  };
//------------------------------------------------------------------	CMyEA
void CMyEA::CMyEA() { }
//------------------------------------------------------------------	~CMyEA
void CMyEA::~CMyEA()
  {
   IndicatorRelease(m_hma);   //   
   IndicatorRelease(m_hbb);//   Bollinger Bands
  }
//------------------------------------------------------------------	Init
bool CMyEA::Init(string smb,ENUM_TIMEFRAMES tf)
  {
   if(!CExpertAdvisor::Init(0,smb,tf)) return(false);                                //   

   //  
   m_risk=Risk; 
   m_tp=TP; 
   m_sl=SL; 
   m_ts=TS;                                           
   m_MAPer=MAPer;
   m_BBPer = BBPer; 
   m_BBDev = BBDev;
   m_hma=iMA(m_smb,PERIOD_D1,m_MAPer,0,MODE_SMA,PRICE_CLOSE);                        //   
   m_hbb=iBands(m_smb,PERIOD_H1,m_BBPer,0,m_BBDev,PRICE_CLOSE);                      //   Bollinger Bands
   if(m_hma==INVALID_HANDLE || m_hbb==INVALID_HANDLE ) return(false);                //    
   m_bInit=true; 
   return(true);                                                                     // " "
  }
//------------------------------------------------------------------	Main
bool CMyEA::Main() //  
  {
   if(!CExpertAdvisor::Main()) return(false); //    

   if(Bars(m_smb,m_tf)<=100) return(false);   //   

   if(!CheckNewBar()) return(true);           //   

                                              //   
   long dir;
   dir=ORDER_TYPE_BUY;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);
   dir=ORDER_TYPE_SELL;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);

   return(true);
  }
//------------------------------------------------------------------	OpenPos
void CMyEA::OpenPosition(long dir)
  {
//   ,  
   if(PositionSelect(m_smb)) return;
//      
   if(dir!=CheckSignal(true)) return;
   double lot=CountLotByRisk(m_sl,m_risk,0);
//    
   if(lot<=0) return;
//  
   DealOpen(dir,lot,m_sl,m_tp);
  }
//------------------------------------------------------------------	ClosePos
void CMyEA::ClosePosition(long dir)
  {
//   ,  
   if(!PositionSelect(m_smb)) return;
//     
   if(dir!=PositionGetInteger(POSITION_TYPE)) return;
//        
   if(dir!=CheckSignal(false)) return;
//  
   m_trade.PositionClose(m_smb,1);
  }
//------------------------------------------------------------------	CheckSignal
long CMyEA::CheckSignal(bool bEntry)
  {
   double ma[4],     //    MA
          bbup[3],   //     Bollinger Bands
          bbdn[3];   //     Bollinger Bands
   
   MqlRates rt[3];   //     3-  
   
   if(CopyRates(m_smb,m_tf,0,3,rt)!=3) //      3-  
     { Print("CopyRates ",m_smb,"   "); return(WRONG_VALUE); }
   //     
   if(CopyBuffer(m_hma,0,0,4,ma)<4 || CopyBuffer(m_hbb,1,0,2,bbup)<2 ||CopyBuffer(m_hbb,2,0,2,bbdn)<2)
     { Print("CopyBuffer -  "); return(WRONG_VALUE); }
   //   MA       Bollinger Bands
   if(rt[1].open<bbdn[1] && rt[1].close>bbdn[1] && ma[1]<ma[2] &&  ma[2]<ma[3])
      return(bEntry ? ORDER_TYPE_BUY:ORDER_TYPE_SELL); //   
   //   MA       Bollinger Bands   
   if(rt[1].open>bbup[1] && rt[2].close<bbup[1]&& ma[1]>ma[2] &&  ma[2]>ma[3])
      return(bEntry ? ORDER_TYPE_SELL:ORDER_TYPE_BUY); //   

   return(WRONG_VALUE); //   
  }

CMyEA ea; //  
//------------------------------------------------------------------	OnInit
int OnInit()
  {
   ea.Init(Symbol(),Period()); //  

                                 //  
// ea.Init(Symbol(), PERIOD_M5); //   
// ea.Init("USDJPY", PERIOD_H2); //     

   return(0);
  }
//------------------------------------------------------------------	OnDeinit
void OnDeinit(const int reason) { }
//------------------------------------------------------------------	OnTick
void OnTick()
  {
   ea.Main(); //   
  }
//+------------------------------------------------------------------+